'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const set = require('just-safe-set');
const multidict = require('multidict');
const skeleton = require('../Accessor/skeleton.js');
const deepCopy = require('../Helpers/deepCopy.js');
const FrailMap = require('../Helpers/FrailMap.js');
const select = require('../Helpers/select.js');
require('just-safe-get');
const crawl = require('./crawl.js');
const normalization = require('./normalization.js');
const persistence = require('./persistence.js');
const utils = require('./utils.js');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

const set__default = /*#__PURE__*/_interopDefault(set);

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _maxAge, _staleWhileRevalidate, _normalizationOptions, _data, _normalizedObjects, _subscriptions, _normalizedSubscriptions, _subscribeNormalized, subscribeNormalized_fn, _notifySubscribers, _selectionAccessors;
class Cache {
  constructor(data, {
    maxAge = Infinity,
    staleWhileRevalidate = 5 * 30 * 1e3,
    normalization: normalization$1
  } = {}) {
    __privateAdd(this, _subscribeNormalized);
    __privateAdd(this, _maxAge, Infinity);
    __privateAdd(this, _staleWhileRevalidate, 0);
    __privateAdd(this, _normalizationOptions, void 0);
    /**
     * The actual data cache. Cache keys are formatted as the first 2 layer of
     * the selection path, e.g. `['query', 'user']` for `query.user`.
     *
     * This enables a cache expiry/eviction strategy based on top-level queries.
     */
    __privateAdd(this, _data, new FrailMap.FrailMap());
    /** Look up table for normalized objects. */
    __privateAdd(this, _normalizedObjects, /* @__PURE__ */ new Map());
    /** Subscription paths and it's listener function. */
    __privateAdd(this, _subscriptions, /* @__PURE__ */ new Map());
    /** Subscription paths that reached a normalized object. */
    __privateAdd(this, _normalizedSubscriptions, new multidict.MultiDict());
    // TODO: Optimization
    // This is pretty inefficient, but maintaining an indexed tree is too much
    // effort right now. Accepting PRs.
    __privateAdd(this, _notifySubscribers, (value) => {
      var _a, _b;
      const listeners = /* @__PURE__ */ new Set();
      const subs = __privateGet(this, _subscriptions);
      const nsubs = __privateGet(this, _normalizedSubscriptions);
      const getId = (_a = this.normalizationOptions) == null ? void 0 : _a.identity;
      for (const [paths, notify] of __privateGet(this, _subscriptions)) {
        for (const path of paths) {
          const parts = path.split(".");
          const node = select.select(value, parts, (node2) => {
            var _a2;
            if ((getId == null ? void 0 : getId(node2)) && utils.isCacheObject(node2)) {
              (_a2 = nsubs.get(node2)) == null ? void 0 : _a2.forEach((notify2) => {
                listeners.add(notify2);
              });
              nsubs.set(node2, notify);
            }
            return node2;
          });
          if (Array.isArray(node) ? node.flat(Infinity).some((item) => item !== void 0) : node !== void 0) {
            listeners.add(notify);
            break;
          }
        }
      }
      if (getId) {
        const norbjs = /* @__PURE__ */ new Set();
        crawl.crawl(value, (node) => {
          if (getId(node) && utils.isCacheObject(node)) {
            norbjs.add(node);
          }
        });
        const resubscribingListeners = /* @__PURE__ */ new Set();
        for (const norbj of norbjs) {
          for (const listener of (_b = nsubs.get(norbj)) != null ? _b : []) {
            listeners.add(listener);
            resubscribingListeners.add(listener);
          }
        }
        for (const listener of resubscribingListeners) {
          for (const [paths, _listener] of subs) {
            if (listener === _listener) {
              __privateMethod(this, _subscribeNormalized, subscribeNormalized_fn).call(this, paths, listener);
            }
          }
        }
      }
      if (listeners.size > 0) {
        const valueSnapshot = deepCopy.deepCopy(value);
        for (const notify of listeners) {
          notify(valueSnapshot);
        }
      }
    });
    /* TODO: Refactor
     *
     * Caching accessors by selections is a mean to retain sub-selections when
     * nullable arrays and objects has null cached, such accessors will return
     * null and no futher selections can be triggered on them.
     *
     * Only when cache value is null, such a cache should be responsible for
     * returning all child selections made last time in this scope.
     *
     * Caching accessors by cache values is broken with normalization enabled.
     * Different selection paths leading to the same normalized object overwrites
     * each other, along with the selection inside.
     *
     * Current workaround is to cache selection children in parents, preventing
     * excessive selection object creation.
     */
    __privateAdd(this, _selectionAccessors, /* @__PURE__ */ new WeakMap());
    __privateSet(this, _maxAge, Math.max(maxAge, 0));
    __privateSet(this, _staleWhileRevalidate, Math.max(staleWhileRevalidate, 0));
    if (normalization$1) {
      __privateSet(this, _normalizationOptions, normalization$1 === true ? normalization.defaultNormalizationHandler : Object.freeze({ ...normalization$1 }));
    }
    if (data) {
      this.restore(data);
    }
  }
  get maxAge() {
    return __privateGet(this, _maxAge);
  }
  get staleWhileRevalidate() {
    return __privateGet(this, _staleWhileRevalidate);
  }
  get normalizationOptions() {
    return __privateGet(this, _normalizationOptions);
  }
  restore(data) {
    var _a;
    const { query, mutation, subscription, normalizedObjects } = (_a = persistence.importCacheSnapshot(data, this.normalizationOptions)) != null ? _a : {};
    __privateSet(this, _normalizedObjects, normalizedObjects != null ? normalizedObjects : new FrailMap.FrailMap());
    __privateSet(this, _data, new FrailMap.FrailMap());
    this.set({ query, mutation, subscription }, { skipNotify: true });
  }
  /** Subscribe to cache changes. */
  subscribe(paths, fn) {
    const pathsSnapshot = Object.freeze([...paths]);
    __privateGet(this, _subscriptions).set(pathsSnapshot, fn);
    __privateMethod(this, _subscribeNormalized, subscribeNormalized_fn).call(this, paths, fn);
    return () => {
      __privateGet(this, _subscriptions).delete(pathsSnapshot);
      __privateGet(this, _normalizedSubscriptions).delete(fn);
    };
  }
  getAccessor(selection, createAccessor) {
    var _a;
    const map = __privateGet(this, _selectionAccessors);
    const accessor = (_a = map.get(selection)) != null ? _a : createAccessor();
    if (!map.has(selection)) {
      map.set(selection, accessor);
    }
    return accessor;
  }
  /**
   * Retrieve cache values by first 2 path segments, e.g. `query.todos` or
   * `mutation.createTodo`.
   */
  get(path, { includeExpired } = {}) {
    var _a;
    const [, type, key, subpath] = (_a = path.match(/^([a-z]+(?:\w*))\.(?:__)?([a-z]+(?:\w*))(.*[^\.])?$/i)) != null ? _a : [];
    if (!type || !key) {
      throw new ReferenceError(
        "Cache path must starts with `${type}.`: " + path
      );
    }
    const cacheKey = `${type}.${key}`;
    let dataContainer = __privateGet(this, _data).get(cacheKey);
    if (dataContainer === void 0) {
      return;
    }
    const { expiresAt, swrBefore } = dataContainer;
    let { data } = dataContainer;
    return {
      data: expiresAt < Date.now() && !includeExpired ? void 0 : subpath ? select.select(data, subpath.slice(1).split(".")) : data,
      expiresAt,
      swrBefore
    };
  }
  /**
   * Merge objects into the current cache, recursively normalize incoming values
   * if normalization is enabled. Notifies cache listeners afterwards.
   *
   * Example value: `{ query: { foo: "bar" } }`
   */
  set(values, { skipNotify = false } = {}) {
    var _a;
    const age = this.maxAge;
    const swr = this.staleWhileRevalidate;
    const now = Date.now();
    if (this.normalizationOptions) {
      values = normalization.deepNormalizeObject(values, {
        ...this.normalizationOptions,
        store: __privateGet(this, _normalizedObjects)
      });
    }
    for (const [type, cacheObjects = {}] of Object.entries(values)) {
      for (const [field, data] of Object.entries(cacheObjects)) {
        const cacheKey = `${type}.${field}`;
        clearTimeout((_a = __privateGet(this, _data).get(cacheKey)) == null ? void 0 : _a.timeout);
        const dataContainer = (
          // Mutation and subscription results should be returned right away for
          // immediate use. Their responses are only meaningful to a cache with
          // normalization enabled, where it updates subscribing clients.
          // We force a short expiration to let it survives the next render,
          // then leave it up for GC.
          type === "mutation" || type === "subscription" ? {
            data,
            expiresAt: now + 100,
            swrBefore: now
          } : {
            data,
            expiresAt: age + now,
            swrBefore: age + swr + now
          }
        );
        if (isFinite(age + swr)) {
          const timeout = setTimeout(
            // Hold on to lexical scope reference, preventing GC from WeakRef.
            () => dataContainer,
            age + swr
          );
          if (typeof timeout === "object") {
            timeout.unref();
          }
          dataContainer.timeout = timeout;
        }
        __privateGet(this, _data).set(cacheKey, dataContainer, { strong: !isFinite(age) });
      }
    }
    if (!skipNotify) {
      __privateGet(this, _notifySubscribers).call(this, values);
    }
  }
  toJSON() {
    const snapshot = (
      // Remove skeletons
      crawl.crawl(
        [...__privateGet(this, _data)].reduce((prev, [key, { data }]) => {
          set__default["default"](prev, key, data);
          return prev;
        }, {}),
        (it, key, obj) => {
          if (skeleton.isSkeleton(it)) {
            delete obj[key];
          }
        }
      )
    );
    if (this.normalizationOptions) {
      return persistence.exportCacheSnapshot(snapshot, this.normalizationOptions);
    } else {
      return snapshot;
    }
  }
}
_maxAge = new WeakMap();
_staleWhileRevalidate = new WeakMap();
_normalizationOptions = new WeakMap();
_data = new WeakMap();
_normalizedObjects = new WeakMap();
_subscriptions = new WeakMap();
_normalizedSubscriptions = new WeakMap();
_subscribeNormalized = new WeakSet();
subscribeNormalized_fn = function(paths, fn) {
  var _a, _b;
  const getId = (_a = this.normalizationOptions) == null ? void 0 : _a.identity;
  if (!getId)
    return;
  const store = __privateGet(this, _normalizedObjects);
  const nsubs = __privateGet(this, _normalizedSubscriptions);
  nsubs.delete(fn);
  for (const path of paths) {
    const [type, field, ...parts] = path.split(".");
    select.select((_b = this.get(`${type}.${field}`)) == null ? void 0 : _b.data, parts, (node) => {
      const id = getId(node);
      if (id && store.has(id) && utils.isCacheObject(node)) {
        nsubs.set(node, fn);
      }
      return node;
    });
  }
};
_notifySubscribers = new WeakMap();
_selectionAccessors = new WeakMap();

exports.Cache = Cache;
