'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const hydrateCache = require('./hydrateCache.js');
const inlineResolved = require('./inlineResolved.js');
const mutate = require('./mutate.js');
const prefetch = require('./prefetch.js');
const prepareRender = require('./prepareRender.js');
const refetch = require('./refetch.js');
const resolved = require('./resolved.js');
const track = require('./track.js');

let deprecationWarningMessage = !process.env.NODE_ENV || process.env.NODE_ENV === "development" ? "[GQty] global query, mutation and subscription is deprecated, use `schema` instead." : void 0;
const createLegacyClient = (options) => {
  const selectionHistory = /* @__PURE__ */ new Set();
  const selectionSubscriptions = /* @__PURE__ */ new Set();
  const prevOnSelect = options.context.onSelect;
  options.context.onSelect = (selection, cache) => {
    if (selectionSubscriptions.size > 0) {
      selectionHistory.add(selection);
    }
    selectionSubscriptions.forEach((fn) => fn(selection, cache));
    prevOnSelect == null ? void 0 : prevOnSelect.call(options.context, selection, cache);
  };
  const methodOptions = {
    ...options,
    subscribeLegacySelections(fn) {
      selectionSubscriptions.add(fn);
      return () => selectionSubscriptions.delete(fn);
    }
  };
  const inlineResolved$1 = inlineResolved.createLegacyInlineResolved(methodOptions);
  return {
    get query() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.query;
    },
    get mutation() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.mutation;
    },
    get subscription() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.subscription;
    },
    resolved: resolved.createLegacyResolved(methodOptions),
    inlineResolved: inlineResolved$1,
    mutate: mutate.createLegacyMutate(methodOptions),
    track: track.createLegacyTrack(methodOptions),
    prefetch: prefetch.createLegacyPrefetch(methodOptions),
    refetch: refetch.createRefetch({
      ...methodOptions,
      selectionHistory,
      inlineResolved: inlineResolved$1
    }),
    prepareRender: prepareRender.createLegacyPrepareRender(methodOptions),
    hydrateCache: hydrateCache.createLegacyHydrateCache(methodOptions),
    subscribeLegacySelections: methodOptions.subscribeLegacySelections
  };
};

exports.createLegacyClient = createLegacyClient;
