import { createLegacyHydrateCache } from './hydrateCache.mjs';
import { createLegacyInlineResolved } from './inlineResolved.mjs';
import { createLegacyMutate } from './mutate.mjs';
import { createLegacyPrefetch } from './prefetch.mjs';
import { createLegacyPrepareRender } from './prepareRender.mjs';
import { createRefetch } from './refetch.mjs';
import { createLegacyResolved } from './resolved.mjs';
import { createLegacyTrack } from './track.mjs';

let deprecationWarningMessage = !process.env.NODE_ENV || process.env.NODE_ENV === "development" ? "[GQty] global query, mutation and subscription is deprecated, use `schema` instead." : void 0;
const createLegacyClient = (options) => {
  const selectionHistory = /* @__PURE__ */ new Set();
  const selectionSubscriptions = /* @__PURE__ */ new Set();
  const prevOnSelect = options.context.onSelect;
  options.context.onSelect = (selection, cache) => {
    if (selectionSubscriptions.size > 0) {
      selectionHistory.add(selection);
    }
    selectionSubscriptions.forEach((fn) => fn(selection, cache));
    prevOnSelect == null ? void 0 : prevOnSelect.call(options.context, selection, cache);
  };
  const methodOptions = {
    ...options,
    subscribeLegacySelections(fn) {
      selectionSubscriptions.add(fn);
      return () => selectionSubscriptions.delete(fn);
    }
  };
  const inlineResolved = createLegacyInlineResolved(methodOptions);
  return {
    get query() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.query;
    },
    get mutation() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.mutation;
    },
    get subscription() {
      deprecationWarningMessage && console.warn(deprecationWarningMessage);
      deprecationWarningMessage = void 0;
      return options.accessor.subscription;
    },
    resolved: createLegacyResolved(methodOptions),
    inlineResolved,
    mutate: createLegacyMutate(methodOptions),
    track: createLegacyTrack(methodOptions),
    prefetch: createLegacyPrefetch(methodOptions),
    refetch: createRefetch({
      ...methodOptions,
      selectionHistory,
      inlineResolved
    }),
    prepareRender: createLegacyPrepareRender(methodOptions),
    hydrateCache: createLegacyHydrateCache(methodOptions),
    subscribeLegacySelections: methodOptions.subscribeLegacySelections
  };
};

export { createLegacyClient };
