'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('../../Error/index.js');

const createLegacyTrack = ({
  cache,
  context: globalContext,
  resolvers: { createResolver },
  subscribeLegacySelections
}) => {
  const track = (fn, { onError, operationName, refetch = false } = {}) => {
    const { context, selections, subscribe } = createResolver({
      cachePolicy: refetch ? "no-cache" : "default",
      operationName
    });
    const resolutionCache = refetch ? context.cache : cache;
    const dataFn = (info) => {
      globalContext.cache = resolutionCache;
      try {
        return fn(info);
      } finally {
        globalContext.cache = cache;
      }
    };
    const unsubscribe = subscribeLegacySelections((selection, cache2) => {
      var _a;
      (_a = context.onSelect) == null ? void 0 : _a.call(context, selection, cache2);
    });
    const data = { current: dataFn({ type: "initial" }) };
    unsubscribe();
    const stop = subscribe({
      onError(error) {
        const theError = index.GQtyError.create(error);
        if (onError) {
          onError(theError);
        } else {
          throw theError;
        }
      },
      onNext() {
        data.current = dataFn({ type: "cache_change" });
      }
    });
    return { data, selections, stop };
  };
  return track;
};

exports.createLegacyTrack = createLegacyTrack;
