import { Cache, CacheDataContainer, CacheGetOptions } from '../Cache';
import type { ScalarsEnumsHash, Schema } from '../Schema';
import type { Selection } from '../Selection';
export type SchemaContext<T extends Record<string, unknown> = Record<string, unknown>> = T & {
    cache: Cache;
    readonly cacheOptions?: CacheGetOptions;
    readonly depthLimit: number;
    readonly scalars: ScalarsEnumsHash;
    /** Root schema for type lookups */
    readonly schema: Readonly<Schema>;
    /** Custom key fields for each object type. */
    readonly typeKeys?: Record<string, string[]>;
    onSelect: (selection: Selection, cache?: CacheDataContainer) => void;
    notifyCacheUpdate: boolean;
    shouldFetch: boolean;
    hasCacheHit: boolean;
    hasCacheMiss: boolean;
};
export type CreateContextOptions = {
    cache: Cache;
    depthLimit: number;
    cachePolicy: RequestCache;
    onSelect?: NonNullable<SchemaContext['onSelect']>;
    scalars: ScalarsEnumsHash;
    schema: Readonly<Schema>;
    typeKeys?: Record<string, string[]>;
};
export declare const createContext: ({ cache, cachePolicy, depthLimit, onSelect, scalars, schema, typeKeys, }: CreateContextOptions) => SchemaContext;
