import type { ExecutionResult } from 'graphql';
import { CloseEvent } from 'ws';
import type { FetchOptions } from '.';
import type { Cache } from '../Cache';
import { GQtyError } from '../Error';
import type { Selection } from '../Selection';
import type { Debugger } from './debugger';
export type FetchSelectionsOptions = {
    cache?: Cache;
    debugger?: Debugger;
    fetchOptions: FetchOptions;
    operationName?: string;
};
export type QueryExtensions = {
    type: string;
    hash: string;
};
export type FetchResult<TData extends Record<string, unknown> = Record<string, unknown>> = Omit<ExecutionResult<TData>, 'errors'> & {
    error?: Error | GQtyError;
};
export declare const fetchSelections: <TData extends Record<string, unknown> = Record<string, unknown>>(selections: Set<Selection>, { cache, debugger: debug, fetchOptions, operationName, }: FetchSelectionsOptions) => Promise<FetchResult<TData>[]>;
export type SubscribeSelectionOptions = FetchSelectionsOptions & {
    onComplete?: () => void;
    onSubscribe?: () => void;
};
export type SubscriptionCallback<TData extends Record<string, unknown>> = (result: FetchResult<TData>) => void;
export type Unsubscribe = () => void;
export declare const subscribeSelections: <TData extends Record<string, unknown> = Record<string, unknown>>(selections: Set<Selection>, fn: SubscriptionCallback<TData>, { cache, debugger: debug, fetchOptions: { subscriber }, operationName, onSubscribe, onComplete, }: SubscribeSelectionOptions) => Unsubscribe;
export declare const isCloseEvent: (input: unknown) => input is CloseEvent;
