'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const retry = require('./retry.js');

class GQtyError extends Error {
  constructor(message, { graphQLErrors, otherError } = {}) {
    super(message);
    this.name = "GQtyError";
    if (graphQLErrors)
      this.graphQLErrors = graphQLErrors;
    if (otherError !== void 0)
      this.otherError = otherError;
  }
  toJSON() {
    return {
      message: this.message,
      graphQLErrors: this.graphQLErrors,
      otherError: this.otherError
    };
  }
  static create(error) {
    let newError;
    if (error instanceof GQtyError) {
      newError = error;
    } else if (error instanceof Error) {
      newError = Object.assign(new GQtyError(error.message), error);
    } else {
      newError = new GQtyError("Unexpected error type", {
        otherError: error
      });
    }
    return newError;
  }
  static fromGraphQLErrors(errors) {
    return new GQtyError(
      errors.length === 1 && errors[0].message || (process.env.NODE_ENV === "production" ? `GraphQL Errors` : "GraphQL Errors, please check .graphQLErrors property"),
      { graphQLErrors: errors }
    );
  }
}

exports.doRetry = retry.doRetry;
exports.GQtyError = GQtyError;
