export type FrailContainer<V> = WeakRef<{
    data: V;
}> | StrongRef<{
    data: V;
}>;
export type SetOptions = {
    /** Use strong reference for this value. */
    strong?: boolean;
};
/** As oppose to WeakRef */
declare class StrongRef<T extends object> {
    #private;
    constructor(data: T);
    deref(): T;
}
/**
 * Strong limbs with a weak grab, a WeakMap that supports scalars.
 *
 * `size` will not be updated until a disposed object is accessed via one of
 * these methods: get, entries, forEach, keys or values.
 */
export declare class FrailMap<K, V> extends Map<K, V> {
    constructor(entries?: readonly (readonly [K, V])[] | null);
    get(key: K): V | undefined;
    has(key: K): boolean;
    set(key: K, value: V, options?: SetOptions): this;
    forEach(callbackfn: (value: V, key: K, map: Map<K, V>) => void, thisArg?: any): void;
    entries(): IterableIterator<[K, V]>;
    keys(): IterableIterator<K>;
    values(): IterableIterator<V>;
    [Symbol.iterator](): IterableIterator<[K, V]>;
    get [Symbol.toStringTag](): string;
    toJSON(): Record<string, any>;
}
export {};
