var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _data;
class StrongRef {
  constructor(data) {
    __privateAdd(this, _data, void 0);
    __privateSet(this, _data, data);
  }
  deref() {
    return __privateGet(this, _data);
  }
}
_data = new WeakMap();
class FrailMap extends Map {
  constructor(entries) {
    super();
    if (entries) {
      for (const [k, v] of entries) {
        this.set(k, v);
      }
    }
  }
  get(key) {
    const ref = super.get(key);
    const val = ref == null ? void 0 : ref.deref();
    if (val === void 0) {
      this.delete(key);
      return;
    }
    return val.data;
  }
  has(key) {
    return super.has(key) && this.get(key) !== void 0;
  }
  set(key, value, options) {
    return super.set(
      key,
      (options == null ? void 0 : options.strong) ? new StrongRef({ data: value }) : new WeakRef({ data: value })
    );
  }
  forEach(callbackfn, thisArg) {
    super.forEach((container, k) => {
      const ref = container.deref();
      if (ref !== void 0) {
        callbackfn.call(thisArg, ref.data, k, this);
      } else {
        this.delete(k);
      }
    });
  }
  entries() {
    const map = /* @__PURE__ */ new Map();
    this.forEach((v, k) => {
      map.set(k, v);
    });
    return map.entries();
  }
  keys() {
    const keys = /* @__PURE__ */ new Set();
    this.forEach((_, k) => {
      keys.add(k);
    });
    return keys.values();
  }
  values() {
    const keys = /* @__PURE__ */ new Set();
    this.forEach((v) => {
      keys.add(v);
    });
    return keys.values();
  }
  [Symbol.iterator]() {
    return this.entries();
  }
  get [Symbol.toStringTag]() {
    return "FrailMap";
  }
  toJSON() {
    const json = {};
    this.forEach((v, k) => {
      json[k] = v;
    });
    return json;
  }
}

export { FrailMap };
