'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const set = require('just-safe-set');
const hash = require('object-hash');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

const set__default = /*#__PURE__*/_interopDefault(set);
const hash__default = /*#__PURE__*/_interopDefault(hash);

const stringifySelectionTree = (tree) => Object.entries(tree).reduce((prev, [key, value]) => {
  const query = typeof value === "object" ? `${key}{${stringifySelectionTree(value)}}` : key;
  if (!prev || prev.endsWith("}") || prev.endsWith("{")) {
    return `${prev}${query}`;
  } else {
    return `${prev} ${query}`;
  }
}, "");
const buildQuery = (selections, operationName) => {
  const root = {};
  const variables = /* @__PURE__ */ new Map();
  const inputDedupe = /* @__PURE__ */ new Map();
  for (const { ancestry } of selections) {
    set__default["default"](
      root,
      ancestry.reduce((prev, s) => {
        if (typeof s.key === "symbol" || typeof s.key === "number" || s.key === "$on") {
          return prev;
        }
        if (s.isUnion) {
          return [...prev, `...on ${s.key}`];
        }
        const key = s.alias ? `${s.alias}:${s.key}` : s.key;
        const input = s.input;
        if (input) {
          if (!inputDedupe.has(input)) {
            const queryInputs = Object.entries(input.values).map(([key2, value]) => {
              const index = variables.size + 1;
              variables.set(`v${index}`, { value, type: input.types[key2] });
              return `${key2}:$v${index}`;
            }).join(" ");
            inputDedupe.set(input, `${key}(${queryInputs})`);
          }
          return [...prev, inputDedupe.get(input)];
        }
        return [...prev, key];
      }, []),
      true
    );
  }
  const branches = Object.entries(root).reduce(
    (prev, [key, value]) => {
      if (key !== "subscription") {
        return [...prev, [key, value]];
      }
      return [
        ...prev,
        ...Object.entries(value).map(
          ([topField, branch]) => ["subscription", { [topField]: branch }]
        )
      ];
    },
    []
  );
  return branches.map(([key, branch]) => {
    const rootKey = key;
    let query = stringifySelectionTree({ [rootKey]: branch });
    if (variables.size > 0) {
      query = query.replace(
        rootKey,
        `${rootKey}(${[...variables.entries()].map(([, { type }], index) => `$v${index + 1}:${type}`).join("")})`
      );
    }
    if (operationName) {
      query = query.replace(rootKey, `${rootKey} ${operationName}`);
    }
    return {
      query,
      variables: variables.size > 0 ? [...variables].reduce(
        (prev, [key2, { value }]) => (prev[key2] = value, prev),
        {}
      ) : void 0,
      operationName,
      extensions: {
        type: rootKey,
        hash: hash__default["default"]({ query, variables })
        // For query dedupe and cache keys
      }
    };
  });
};

exports.buildQuery = buildQuery;
