import memoize from 'just-memoize';
import hash from 'object-hash';
import { GQtyError } from '../Error/index.mjs';

const createSymbol = Symbol();
const aliasGenerator = {
  seq: 0,
  map: /* @__PURE__ */ new WeakMap(),
  hash: memoize((...args) => hash(args).replace(/^\d+/, "")),
  get(key, input) {
    var _a;
    const hash2 = this.hash({ key, ...input });
    if (hash2)
      return hash2;
    const seq = (_a = this.map.get(input)) != null ? _a : this.seq++;
    if (seq > Number.MAX_SAFE_INTEGER) {
      throw new GQtyError(`selection alias fallback overflow`);
    }
    this.map.set(input, seq);
    return `alias${seq}`;
  }
};
class Selection {
  constructor(key, { input, alias, isUnion = false, parent } = {}, token) {
    this.key = key;
    this.cacheKeys = [];
    this.children = /* @__PURE__ */ new Map();
    var _a, _b, _c;
    if (token !== createSymbol) {
      throw new GQtyError(`Use Selection.createRoot() instead.`);
    }
    this.alias = alias;
    this.input = input;
    this.isUnion = isUnion;
    this.parent = parent;
    this.root = (_a = parent == null ? void 0 : parent.root) != null ? _a : this;
    if (typeof key === "number" || key === "$on" || (parent == null ? void 0 : parent.key) === "$on") {
      this.cacheKeys = (_b = parent == null ? void 0 : parent.cacheKeys) != null ? _b : [];
    } else {
      this.cacheKeys = ((_c = parent == null ? void 0 : parent.cacheKeys) != null ? _c : []).concat(this.alias || key);
    }
  }
  /** The selection path from root the leaf as an array. */
  get ancestry() {
    const ancestry = [];
    let current = this;
    while (current) {
      ancestry.unshift(current);
      current = current.parent;
    }
    return ancestry;
  }
  static createRoot(key) {
    return new Selection(key, {}, createSymbol);
  }
  getChild(key, options) {
    var _a, _b, _c;
    const alias = (_b = options == null ? void 0 : options.alias) != null ? _b : (options == null ? void 0 : options.input) ? aliasGenerator.get(key, options.input).slice(0, (_a = options.aliasLength) != null ? _a : 5) : void 0;
    const hashKey = alias != null ? alias : key.toString();
    const selection = (_c = this.children.get(hashKey)) != null ? _c : new Selection(key, { ...options, alias, parent: this }, createSymbol);
    this.children.set(hashKey, selection);
    return selection;
  }
  getLeafNodes() {
    const result = /* @__PURE__ */ new Set();
    const stack = /* @__PURE__ */ new Set([this]);
    for (const selection of stack) {
      if (selection.children.size === 0) {
        result.add(selection);
      } else {
        for (const [, child] of selection.children)
          stack.add(child);
      }
    }
    return result;
  }
  toJSON() {
    return this.ancestry.map(({ key, isUnion, input }) => {
      if (isUnion) {
        return [key, { isUnion }];
      } else if (input) {
        return [key, { input }];
      } else {
        return [key];
      }
    });
  }
  fromJSON(json) {
    let node = this;
    for (const [key, options] of json) {
      node = node.getChild(key, options);
    }
    return node;
  }
}

export { Selection };
